@@ -2,6 +2,14 @@
 #
 
 anova.sarlm <- function(object, ...) {
+#    warning("Method anova moved to the spatialreg package")
+#    if (!requireNamespace("spatialreg", quietly=TRUE))
+#      stop("install the spatialreg package")
+#    if (requireNamespace("spatialreg", quietly=TRUE)) {
+#      return(anova(object=object, ...))
+#    }
+    warning("install the spatialreg package")
+#  if (FALSE) {
     if (length(list(object, ...)) > 1L) {
         getResponseFormula <- function (object) 
         {
@@ -72,5 +80,6 @@ anova.sarlm <- function(object, ...) {
         return(res)
     }
 }
+#}
 
 
