@@ -1,9 +1,10 @@
-# Copyright 2001-2014 by Roger S. Bivand. 
+# Copyright 2001-2021 by Roger S. Bivand. 
 # Upgrade to sp classes February 2007
 # Added RANN April 2010
 # nn() retired 130722
+# shift to dbscan 210317 #53
 
-knearneigh <- function(x, k=1, longlat=NULL, RANN=TRUE)
+knearneigh <- function(x, k=1, longlat=NULL, use_kd_tree=TRUE)
 {
     if (inherits(x, "SpatialPoints")) {
         if ((is.null(longlat) || !is.logical(longlat)) 
@@ -33,7 +34,9 @@ knearneigh <- function(x, k=1, longlat=NULL, RANN=TRUE)
     if (is.null(longlat) || !is.logical(longlat)) longlat <- FALSE
     if (!is.numeric(x)) stop("knearneigh: data non-numeric")
     if (!is.matrix(x)) stop("knearneigh: data not in matrix form")
-    stopifnot(ncol(x) == 2)
+# https://github.com/r-spatial/spdep/issues/38
+    if (ncol(x) > 2 && longlat == TRUE)
+        stop("ncol(x) > 2 only permitted if longlat is FALSE")
     if (any(is.na(x))) stop("knearneigh: data include NAs")
     if (longlat) {
         bb <- bbox(x)
@@ -43,30 +46,23 @@ knearneigh <- function(x, k=1, longlat=NULL, RANN=TRUE)
     if (!is.double(x)) storage.mode(x) <- "double"
     np <- nrow(x)
     dimension <- ncol(x)
-    if (dimension != 2) stop("knearneigh: only 2D data accepted")
+# https://github.com/r-spatial/spdep/issues/38
+#    if (dimension != 2) stop("knearneigh: only 2D data accepted")
     if (k >= np) stop("knearneigh: fewer data points than k")
     if (k > (np/3)) warning("k greater than one-third of the number of data points")
 # modified 140117 to handle zerodist points
 # (previous fix only worked for pairs and k>1)
-    zd <- zerodist(SpatialPoints(x))
-    if (!(nrow(zd) == 0)) warning("knearneigh: identical points found")
-    useRANN <- RANN
-    if (longlat) useRANN <- FALSE
-    if (nrow(zd) > 0) useRANN <- FALSE
-    if (useRANN && !requireNamespace("RANN", quietly = TRUE)) useRANN <- FALSE
-    if (useRANN) {
-#        xx <- cbind(x, out=rep(0, nrow(x)))
-#        out <- as.matrix(nn(xx, p=k)$nn.idx)
-# nn() retired 130722
-# modified 130913 to handle zerodist points
-#        zd <- zerodist(SpatialPoints(x))
-#        if (nrow(zd) == 0) {
-        out <- RANN::nn2(x, x, k=k+1)$nn.idx[,-1,drop=FALSE]
-#        } else {
-#            out0 <- nn2(x, x, k=k+1)
-#            out <- t(sapply(1:np, function(i) out0$nn.idx[i,
-#                -which(out0$nn.idx[i,] == i), drop=FALSE]))
-#        }
+# https://github.com/r-spatial/spdep/issues/38
+    zd <- any(duplicated(x)) #zerodist(SpatialPoints(x))
+# kNN() handles duplicate points
+    if (zd)  warning("knearneigh: identical points found")
+    if (longlat) use_kd_tree <- FALSE
+    if (zd) use_kd_tree <- FALSE
+    if (use_kd_tree && !requireNamespace("dbscan", quietly = TRUE)) use_kd_tree <- FALSE
+# https://github.com/r-spatial/spdep/issues/38
+    if (dimension > 2 && !use_kd_tree) stop("dbscan required for ncol(x) > 2")
+    if (use_kd_tree) {
+        out <- dbscan::kNN(x, k=k)$id
         dimnames(out) <- NULL
         res <- list(nn=out, np=np, k=k, dimension=dimension, x=x)
     } else {
