@@ -1,4 +1,4 @@
-# Copyright 2001-2010 by Roger Bivand 
+# Copyright 2001-2021 by Roger Bivand 
 #
 #
 # Modified by Micah Altman 2010
@@ -6,7 +6,7 @@
 
 
 poly2nb <- function(pl, row.names=NULL, snap=sqrt(.Machine$double.eps),
-	queen=TRUE, useC=TRUE, foundInBox=NULL) {
+	queen=TRUE, useC=TRUE, foundInBox=NULL, small_n=500) {
         verbose <- get("verbose", envir = .spdepOptions)
         .ptime_start <- proc.time()
         sf <- NULL
@@ -19,8 +19,10 @@ poly2nb <- function(pl, row.names=NULL, snap=sqrt(.Machine$double.eps),
                 pl <- sf::st_geometry(pl)
             }
             if (inherits(pl, "sfc")) {
-                if (length(grep("POLYGON", class(pl))) == 0L)
-                    stop("Polygon geometries required")
+                if (length(grep("POLYGON", class(pl)[1])) == 0L)
+                    pl <- try(st_cast(pl, "MULTIPOLYGON"), silent=TRUE)
+                    if (inherits(pl, "try-error")) 
+                      stop("Polygon geometries required")
                 if (attr(pl, "n_empty") > 0L) 
                     stop("Empty geometries found")
                 sf <- TRUE
@@ -53,14 +55,14 @@ poly2nb <- function(pl, row.names=NULL, snap=sqrt(.Machine$double.eps),
             bsnap <- snap
         }
         vbsnap <- c(-bsnap, snap)
-        if (verbose)
-            cat("handle IDs:", (proc.time() - .ptime_start)[3], "\n")
+        if (verbose) cat("handle IDs:", (proc.time() - .ptime_start)[3], "\n")
         .ptime_start <- proc.time()
 
         if(sf) {
             xpl0 <- as.data.frame(sf::st_coordinates(pl))
             xpl <- unname(split(xpl0[,1:2], xpl0[, length(xpl0)]))
-            xxpl <- lapply(xpl, function(x) do.call("cbind", x))
+# https://github.com/r-spatial/spdep/issues/50
+            xxpl <- lapply(xpl, function(x) do.call("cbind", x[-1,]))
         } else {
             xpl <- slot(pl, "polygons")
             xxpl <- vector(mode="list", length=length(xpl))
@@ -78,120 +80,79 @@ poly2nb <- function(pl, row.names=NULL, snap=sqrt(.Machine$double.eps),
         }))
         if (verbose)
             cat("massage polygons:", (proc.time() - .ptime_start)[3], "\n")
+
         .ptime_start <- proc.time()
-#poly2bbs <- function(pl) {
-#    t(sapply(slot(pl, "polygons"), bbox))
-#}
-        genBBIndex<-function(bb) { 
-            n <- nrow(bb)
-            bxv <- as.vector(bb[,c(1,3)])
-            byv <- as.vector(bb[,c(2,4)])
-            obxv <- order(bxv)
-            rbxv <- c(1:(n*2))[obxv]
-            mbxv <- match(1:(n*2),obxv)
-            obyv <- order(byv)
-            rbyv <- c(1:(n*2))[obyv]
-            mbyv <- match(1:(n*2),obyv)
-
-            return(list(bb=bb, bxv=bxv, byv=byv, obxv=obxv, obyv=obyv, 
-                mbxv=mbxv, mbyv=mbyv, rbyv=rbyv, rbxv=rbxv))
-        }
-
-
 	dbsnap <- as.double(bsnap)
         dsnap <- as.double(snap)
         if (is.null(foundInBox)) {
-	    BBindex <- genBBIndex(bb)
-        if (verbose)
-            cat("size of BBindex:", object.size(BBindex), "\n")
+            if (n < small_n) {
+                genBBIndex <- function(bb) { 
+                    n <- nrow(bb)
+                    bxv <- as.vector(bb[,c(1,3)])
+                    byv <- as.vector(bb[,c(2,4)])
+                    obxv <- order(bxv)
+                    rbxv <- c(1:(n*2))[obxv]
+                    mbxv <- match(1:(n*2),obxv)
+                    obyv <- order(byv)
+                    rbyv <- c(1:(n*2))[obyv]
+                    mbyv <- match(1:(n*2),obyv)
 
-        } else {
-            stopifnot(is.list(foundInBox))
-            stopifnot(length(foundInBox) == (n-1L))
-            stopifnot(all(unlist(sapply(foundInBox,
-                function(x) {if(!is.null(x)) is.integer(x)}))))
-            nfIBB <- sum(sapply(foundInBox, length))
+                    return(list(bb=bb, bxv=bxv, byv=byv, obxv=obxv, obyv=obyv, 
+                        mbxv=mbxv, mbyv=mbyv, rbyv=rbyv, rbxv=rbxv))
+                }
+	        BBindex <- genBBIndex(bb)
+                if (verbose) cat("size of BBindex:", object.size(BBindex), "\n")
+                foundInBox <- lapply(1:(n-1), function(i) findInBox(i, BBindex))
+            } else {
+                if (!sf) pl <- st_as_sfc(pl)
+                fB1 <- st_intersects(st_as_sfc(lapply(pl, function(x) {
+                    st_as_sfc(st_bbox(x))[[1]]})))
+                fB1a <- lapply(seq_along(fB1), function(i) {
+                    fB1[[i]][fB1[[i]] > i]})
+                foundInBox <- fB1a[-length(fB1a)]
+            }
+            if (verbose) cat("findInBox:", (proc.time() - .ptime_start)[3])
+        }
+        stopifnot(is.list(foundInBox))
+        stopifnot(length(foundInBox) == (n-1L))
+        stopifnot(all(unlist(sapply(foundInBox,
+            function(x) {if(!is.null(x)) is.integer(x)}))))
+        nfIBB <- sum(sapply(foundInBox, length))
+        if (verbose) {
+            cat(" list size", nfIBB, "\n")
+            cat("generate foundInBox:", (proc.time() - .ptime_start)[3], "\n")
         }
-        if (verbose)
-            cat("generate BBs:", (proc.time() - .ptime_start)[3], "\n")
         .ptime_start <- proc.time()
 
-#	nrs <- integer(n)
-#	for (i in 1:n) {
-#		pl[[i]] <- na.omit(pl[[i]][-1,])
-#		nrs[i] <- as.integer(nrow(pl[[i]]))
-#		pl[[i]] <- as.double(pl[[i]])
-#	}
-	
-#        findInBox <- function(i, sp, bigger=TRUE) {
-#	    n <- dim(sp$bb)[1]
-#	
-#	# ! i1 > j3 --> i1 <= j3
-#	    tmp1 <- sp$rbxv[sp$mbxv[i]:(n*2)] 
-#	    tmp1 <- tmp1[which(tmp1>n)] - n
-#	# ! i2 > j4 --> i2 <= bj4
-#	    tmp2 <- sp$rbyv[sp$mbyv[i]:(n*2)] 
-#	    tmp2 <- tmp2[which(tmp2>n)] - n
-#	# ! i3 < j1 -> i3 >= j1
-#	    tmp3 <- sp$rbxv[1:sp$mbxv[i+n]] 
-#	    tmp3 <- tmp3[which(tmp3<=n)]
-#	# ! i4 < j2 -> i4 >= j2
-#	    tmp4 <- sp$rbyv[1:sp$mbyv[i+n]] 
-#	    tmp4 <- tmp4[which(tmp4<=n)]
-#	    result <- intersect(intersect(tmp1,tmp2), intersect(tmp3,tmp4))
-#	    if (bigger) {
-#		result <- result[which(result>i)]
-#	    }
-#	    return(sort(result))
-#        }
-
-	polypoly2 <- function(poly1, nrs1, poly2, nrs2, snap) {
+	criterion <- ifelse(queen, 0, 1)
+        if (useC) {
+              ans <- .Call("poly_loop2", as.integer(n), foundInBox, bb, xxpl,
+                as.integer(nrs), as.double(dsnap), as.integer(criterion),
+                as.integer(nfIBB), PACKAGE="spdep")
+        } else {
+	    polypoly2 <- function(poly1, nrs1, poly2, nrs2, snap) {
 		if (any(nrs1 == 0 || nrs2 == 0)) return(0L)
 		res <- .Call("polypoly", poly1, nrs1, poly2, 
 			nrs2, snap, PACKAGE="spdep")
 		res
-	}
+	    }
 
-        if (is.null(foundInBox)) {
-            foundInBox <- lapply(1:(n-1), function(i) findInBox(i, BBindex))
-            if (verbose) {
-                cat("findInBox:", (proc.time() - .ptime_start)[3])
-            }
-          nfIBB <- sum(sapply(foundInBox, length))
-          if (verbose) cat(" list size", nfIBB, "\n")
-
-          .ptime_start <- proc.time()
-        }
-	criterion <- ifelse(queen, 0, 1)
-        if (useC) {
-#            if (justC) {
-              ans <- .Call("poly_loop2", as.integer(n), foundInBox, bb, xxpl,
-                as.integer(nrs), as.double(dsnap), as.integer(criterion),
-                as.integer(nfIBB), PACKAGE="spdep")
-#            } else {
-#              ans <- .Call("poly_loop", as.integer(n), i_findInBox, bb, pl, 
-#                nrs, as.double(dsnap), as.integer(criterion), as.integer(10),
-#                PACKAGE="spdep")
-#            }
-        } else {
 	    ans <- vector(mode="list", length=n)
 	    for (i in 1:n) ans[[i]] <- integer(0)
 	    for (i in 1:(n-1)) {
-		#for (j in (i+1):n) {
-#		for (j in findInBox(i,BBindex)) {
 		for (j in foundInBox[[i]]) {
-			jhit <- .Call("spOverlap", bb[i,], 
-				bb[j,], PACKAGE="spdep")
-			if (jhit > 0) {
-			    khit <- 0
-			    khit <- polypoly2(xxpl[[i]], nrs[i], xxpl[[j]], 
-				nrs[j], dsnap)
+	            jhit <- .Call("spOverlap", bb[i,], 
+			bb[j,], PACKAGE="spdep")
+		    if (jhit > 0) {
+		        khit <- 0
+		        khit <- polypoly2(xxpl[[i]], nrs[i], xxpl[[j]], 
+			    nrs[j], dsnap)
 
-			    if (khit > criterion) {
-				ans[[i]] <- c(ans[[i]], j)
-				ans[[j]] <- c(ans[[j]], i)
-			    }
-			}
+		        if (khit > criterion) {
+			    ans[[i]] <- c(ans[[i]], j)
+			    ans[[j]] <- c(ans[[j]], i)
+		        }
+		    }
 		}
 	    }
 	    for (i in 1:n) {
@@ -199,17 +160,16 @@ poly2nb <- function(pl, row.names=NULL, snap=sqrt(.Machine$double.eps),
                 if (length(ans[[i]]) > 1L) ans[[i]] <- sort(ans[[i]])
             }
         }
-        if (verbose)
-            cat("work loop:", (proc.time() - .ptime_start)[3], "\n")
+        if (verbose) cat("work loop:", (proc.time() - .ptime_start)[3], "\n")
         .ptime_start <- proc.time()
+
 	class(ans) <- "nb"
 	attr(ans, "region.id") <- regid
 	attr(ans, "call") <- match.call()
 	if (queen) attr(ans, "type") <- "queen"
 	else attr(ans, "type") <- "rook"
 	ans <- sym.attr.nb(ans)
-        if (verbose)
-            cat("done:", (proc.time() - .ptime_start)[3], "\n")
+        if (verbose) cat("done:", (proc.time() - .ptime_start)[3], "\n")
         .ptime_start <- proc.time()
 	ans
 }	
