@@ -4,6 +4,14 @@
 
 spBreg_lag <- function(formula, data = list(), listw, na.action, Durbin, type,
     zero.policy=NULL, control=list()) {
+#    .Deprecated("spatialreg::spBreg_lag", msg="Function spBreg_lag moved to the spatialreg package")
+#    if (!requireNamespace("spatialreg", quietly=TRUE))
+#      stop("install the spatialreg package")
+#    if (requireNamespace("spatialreg", quietly=TRUE)) {
+#      return(spatialreg::spBreg_lag(formula=formula, data=data, listw=listw, na.action=na.action, Durbin=Durbin, type=type, zero.policy=zero.policy, control=control))
+#    }
+    warning("install the spatialreg package")
+#  if (FALSE) {
     timings <- list()
     .ptime_start <- proc.time()
 #control
@@ -87,7 +95,7 @@ spBreg_lag <- function(formula, data = list(), listw, na.action, Durbin, type,
             dmf <- lm(Durbin, data, na.action=na.action, 
 	        method="model.frame")
             fx <- try(model.matrix(Durbin, dmf), silent=TRUE)
-            if (class(fx) == "try-error") 
+            if (inherits(fx, "try-error")) 
                 stop("Durbin variable mis-match")
             WX <- create_WX(fx, listw, zero.policy=zero.policy,
                 prefix=prefix)
@@ -374,10 +382,19 @@ spBreg_lag <- function(formula, data = list(), listw, na.action, Durbin, type,
 
 #output mcmc object
 }
+#}
 
 
 impacts.MCMC_sar_g <- function(obj, ..., tr=NULL, listw=NULL, evalues=NULL,
     Q=NULL) {
+#    .Deprecated("spatialreg::impacts.MCMC_sar_g", msg="Method impacts.MCMC_sar_g moved to the spatialreg package")
+#    if (!requireNamespace("spatialreg", quietly=TRUE))
+#      stop("install the spatialreg package")
+#    if (requireNamespace("spatialreg", quietly=TRUE)) {
+#      return(spatialreg::impacts.MCMC_sar_g(obj=obj, ..., tr=tr, listw=listw, evalues=evalues, Q=Q))
+#    }
+    warning("install the spatialreg package")
+#  if (FALSE) {
     if (is.null(listw) && !is.null(attr(obj, "listw_style")) && 
         attr(obj, "listw_style") != "W")
         stop("Only row-standardised weights supported")
@@ -466,6 +483,7 @@ impacts.MCMC_sar_g <- function(obj, ..., tr=NULL, listw=NULL, evalues=NULL,
 
 
 }
+#}
 
 
 # translated from Matlab code sem_g.m in the Spatial Econometrics toolbox by
@@ -474,6 +492,14 @@ impacts.MCMC_sar_g <- function(obj, ..., tr=NULL, listw=NULL, evalues=NULL,
 
 spBreg_err <- function(formula, data = list(), listw, na.action, Durbin, etype,
     zero.policy=NULL, control=list()) {
+#    .Deprecated("spatialreg::spBreg_err", msg="Function spBreg_err moved to the spatialreg package")
+#    if (!requireNamespace("spatialreg", quietly=TRUE))
+#      stop("install the spatialreg package")
+#    if (requireNamespace("spatialreg", quietly=TRUE)) {
+#      return(spatialreg::spBreg_err(formula=formula, data=data, listw=listw, na.action=na.action, Durbin=Durbin, etype=etype, zero.policy=zero.policy, control=control))
+#    }
+    warning("install the spatialreg package")
+#  if (FALSE) {
     timings <- list()
     .ptime_start <- proc.time()
 #control
@@ -553,7 +579,7 @@ spBreg_err <- function(formula, data = list(), listw, na.action, Durbin, etype,
             dmf <- lm(Durbin, data, na.action=na.action, 
 	        method="model.frame")
             fx <- try(model.matrix(Durbin, dmf), silent=TRUE)
-            if (class(fx) == "try-error") 
+            if (inherits(fx, "try-error")) 
                 stop("Durbin variable mis-match")
             WX <- create_WX(fx, listw, zero.policy=zero.policy,
                 prefix=prefix)
@@ -588,7 +614,7 @@ spBreg_err <- function(formula, data = list(), listw, na.action, Durbin, etype,
     }
 #        x <- cbind(x, WX)
 #        rm(WX)
-#    } else if (type != "lag") stop("No such type:", type)
+#     else if (type != "lag") stop("No such type:", type)
     lm.base <- lm(y ~ x - 1) # doesn't like dgCMatrix
     aliased <- is.na(coefficients(lm.base))
     cn <- names(aliased)
@@ -921,9 +947,19 @@ spBreg_err <- function(formula, data = list(), listw, na.action, Durbin, etype,
 #output mcmc object
 
 }
+#}
 
 impacts.MCMC_sem_g <- function(obj, ..., tr=NULL, listw=NULL, evalues=NULL,
     Q=NULL) {
+#    .Deprecated("spatialreg::impacts.MCMC_sem_g", msg="Method impacts.MCMC_sem_g moved to the spatialreg package")
+#    if (!requireNamespace("spatialreg", quietly=TRUE))
+#      stop("install the spatialreg package")
+#    if (requireNamespace("spatialreg", quietly=TRUE)) {
+#      return(spatialreg::impacts.MCMC_sem_g(obj=obj, ..., tr=tr, listw=listw, evalues=evalues,
+#    Q=Q))
+#    }
+    warning("install the spatialreg package")
+#  if (FALSE) {
     emixedImps <- attr(obj, "emixedImps")
     if (is.null(emixedImps)) {
         stop("No indirect impacts, use summary()")
@@ -932,6 +968,7 @@ impacts.MCMC_sem_g <- function(obj, ..., tr=NULL, listw=NULL, evalues=NULL,
     k <- attr(obj, "k")
     impactsWX(emixedImps, n, k, type="SDEM", method="MCMC")
 }
+#}
 
 
 # translated from Matlab code sac_g.m in the Spatial Econometrics toolbox by
@@ -940,6 +977,14 @@ impacts.MCMC_sem_g <- function(obj, ..., tr=NULL, listw=NULL, evalues=NULL,
 
 spBreg_sac <- function(formula, data = list(), listw, listw2=NULL, na.action, 
     Durbin, type, zero.policy=NULL, control=list()) {
+#    .Deprecated("spatialreg::spBreg_sac", msg="Function spBreg_sac moved to the spatialreg package")
+#    if (!requireNamespace("spatialreg", quietly=TRUE))
+#      stop("install the spatialreg package")
+#    if (requireNamespace("spatialreg", quietly=TRUE)) {
+#      return(spatialreg::spBreg_sac(formula=formula, data=data, listw=listw, listw2=listw2, na.action=na.action, Durbin=Durbin, type=type, zero.policy=zero.policy, control=control))
+#    }
+    warning("install the spatialreg package")
+#  if (FALSE) {
     timings <- list()
     .ptime_start <- proc.time()
 #control
@@ -1016,7 +1061,7 @@ spBreg_sac <- function(formula, data = list(), listw, listw2=NULL, na.action,
     m <- ncol(x)
     dvars <- c(NCOL(x), 0L)
 
-#    if (type == "Durbin") {
+#    if (type == "Durbin") 
 #        WX <- create_WX(x, listw, zero.policy=zero.policy, prefix="lag")
 #FIXME
     if (is.formula(Durbin) || isTRUE(Durbin)) {
@@ -1028,7 +1073,7 @@ spBreg_sac <- function(formula, data = list(), listw, listw2=NULL, na.action,
             dmf <- lm(Durbin, data, na.action=na.action, 
 	        method="model.frame")
             fx <- try(model.matrix(Durbin, dmf), silent=TRUE)
-            if (class(fx) == "try-error") 
+            if (inherits(fx, "try-error")) 
                 stop("Durbin variable mis-match")
             WX <- create_WX(fx, listw, zero.policy=zero.policy,
                 prefix=prefix)
@@ -1331,10 +1376,20 @@ spBreg_sac <- function(formula, data = list(), listw, listw2=NULL, na.action,
 
 #output mcmc object
 }
+#}
 
 
 impacts.MCMC_sac_g <- function(obj, ..., tr=NULL, listw=NULL, evalues=NULL,
     Q=NULL) {
+#    .Deprecated("spatialreg::impacts.MCMC_sac_g", msg="Method impacts.MCMC_sac_g moved to the spatialreg package")
+#    if (!requireNamespace("spatialreg", quietly=TRUE))
+#      stop("install the spatialreg package")
+#    if (requireNamespace("spatialreg", quietly=TRUE)) {
+#      return(spatialreg::impacts.MCMC_sac_g(obj=obj, ..., tr=tr, listw=listw, evalues=evalues,
+#    Q=Q))
+#    }
+    warning("install the spatialreg package")
+#  if (FALSE) {
     obj_lag <- obj[, -(which(colnames(obj) == "lambda"))]
     attributes(obj_lag) <- c(attributes(obj_lag),
         attributes(obj)[5:13])
@@ -1353,3 +1408,4 @@ impacts.MCMC_sac_g <- function(obj, ..., tr=NULL, listw=NULL, evalues=NULL,
     }
     res
 }
+#}
